/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <stdlib.h>
#include <cstring>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include "../config.h"
#include "Config.h"
#include "Exception.h"
#include "Util.h"

/* These should match Config::GPSSource_enum */
char GPSSource_text[][16]={
		"gpsd",
		"serial",
		"simulated",
		""
};

Config::Config() {
	SetDefaults();
}

Config::~Config() {
}

bool Config::Load() {
	std::ifstream ifs;
	std::string Line,CfgFilename;
	int LineCount=0;

	CfgFilename=UserDataDir+std::string("/")+
		std::string(PACKAGE)+std::string(".conf");
	Util::ShellExpand(&CfgFilename);
	ifs.open(CfgFilename.c_str());

	if (ifs.fail()) {
		std::cout<<"Unable to open "<<CfgFilename<<", using defaults."<<std::endl;

		/* Create the user data directory if it doesn't exist */
		struct stat statdata;

		std::string UserDataDirExp(UserDataDir);
		Util::ShellExpand(&UserDataDirExp);
		errno=0;
		if (stat(UserDataDirExp.c_str(),&statdata)!=0) {
			if (errno==ENOENT) {
				if (Util::MkDir(UserDataDirExp)!=0) {
					return false;
				}
			}
		} else {
			if (!S_ISDIR(statdata.st_mode)) {
				Exception(UserDataDirExp+" is not a directory",Exception::LEVEL_ERROR).Show();
			}
		}
		return false;
	}

	char *token,*value,*str;
	while (!ifs.eof() && !ifs.fail()) {
		getline(ifs,Line);
		LineCount++;
		/* Ignore blank lines and lines that start with '#' */
		if (!Line.empty() && (Line[0]!='#') ) {
			str=(char*)malloc(Line.length()+1);
			strncpy(str,Line.c_str(),Line.length()+1);
			if (str) {
				token=strtok(str,"=");
				value=strtok(NULL,"=");
				if (token && value) {
					if (strcasecmp(token,"ConfigVer")==0)
					{
						ConfigVer=atoi(value);
					} else if (strcasecmp(token,"GPSSource")==0)
					{
						if (!SetGPSSource(value))
						{
							fprintf(stderr,"Invalid GPS Source '%s'\n",value);
						}
					} else if (strcasecmp(token,"SerialPortDevice")==0)
					{
						SerialPortDevice=value;
					} else if (strcasecmp(token,"SerialPortBaud")==0)
					{
						SerialPortBaud=atoi(value);
					} else if (strcasecmp(token,"AutoActivateGPS")==0)
					{
						AutoActivateGPS=MkBool(value);
					} else if (strcasecmp(token,"AutoStartgpsd")==0)
					{
						AutoStartgpsd=MkBool(value);
					} else if (strcasecmp(token,"GPSUpdateFrequency")==0)
					{
						GPSUpdateFrequency=atoi(value);
					} else if (strcasecmp(token,"PDFViewer")==0)
					{
						PDFViewer=value;
					} else if (strcasecmp(token,"ImageViewer")==0)
					{
						ImageViewer=value;
					} else if (strcasecmp(token,"PackageDataDir")==0)
					{
						PackageDataDir=value;
					} else if (strcasecmp(token,"UserDataDir")==0)
					{
						UserDataDir=value;
					} else if (strcasecmp(token,"METARSourceURL")==0)
					{
						METARSourceURL=value;
					} else if (strcasecmp(token,"SaveConfigOnExit")==0)
					{
						SaveConfigOnExit=MkBool(value);
					} else if (strcasecmp(token,"CenterOnGPS")==0)
					{
						CenterOnGPS=MkBool(value);
					} else if (strcasecmp(token,"gpsdHost")==0)
					{
						gpsdHost=value;
					} else if (strcasecmp(token,"gpsdPort")==0)
					{
						gpsdPort=value;
					} else if (strcasecmp(token,"gpsdCmd")==0)
					{
						gpsdCmd=value;
					} else if (strcasecmp(token,"DataStoreDir")==0)
					{
						DataStoreDir=value;
					} else if (strcasecmp(token,"dTPPImportDir")==0)
					{
						dTPPImportDir=value;
					} else if (strcasecmp(token,"SectionalsImportDir")==0)
					{
						SectionalsImportDir=value;
					} else if (strcasecmp(token,"DigitalAeroInfoImportDir")==0)
					{
						DigitalAeroInfoImportDir=value;
					} else if (strcasecmp(token,"NationalFlightDBImportDir")==0)
					{
						NationalFlightDBImportDir=value;
					} else if (strcasecmp(token,"XPlane900DataImportDir")==0)
					{
						XPlane900DataImportDir=value;
					} else if (strcasecmp(token,"AutoSelectChart")==0)
					{
						AutoSelectChart=MkBool(value);
					} else if (strcasecmp(token,"GPSSimLon")==0)
					{
						GPSSimLon=atof(value);
					} else if (strcasecmp(token,"GPSSimLat")==0)
					{
						GPSSimLat=atof(value);
					} else if (strcasecmp(token,"GPSSimSpd")==0)
					{
						GPSSimSpd=atof(value);
					} else if (strcasecmp(token,"GPSSimTrk")==0)
					{
						GPSSimTrk=atof(value);
					} else if (strcasecmp(token,"GPSSimAlt")==0)
					{
						GPSSimAlt=atof(value);
					} else if (strcasecmp(token,"DisableSS")==0)
					{
						DisableSS=MkBool(value);
					} else if (strcasecmp(token,"DisablePM")==0)
					{
						DisablePM=MkBool(value);
					} else if (strcasecmp(token,"ChartScale")==0)
					{
						ChartScale=atof(value);
					} else if (strcasecmp(token,"OpenChartFile")==0)
					{
						OpenChartFile=value;
					} else if (strcasecmp(token,"OpenRouteFile")==0)
					{
						OpenRouteFile=value;
					} else if (strcasecmp(token,"OpenAircraftFile")==0)
					{
						OpenAircraftFile=value;
					} else if (strcasecmp(token,"ChartXOffset")==0)
					{
						ChartXOffset=atoi(value);
					} else if (strcasecmp(token,"ChartYOffset")==0)
					{
						ChartYOffset=atoi(value);
					} else if (strcasecmp(token,"ShowWelcomeOnStartup")==0)
					{
						ShowWelcomeOnStartup=MkBool(value);
					} else if (strcasecmp(token,"METARHistory")==0)
					{
						SetMETARHistory(atoi(value));
					} else if (strcasecmp(token,"ShapeFile")==0)
					{
						ShapeFile=value;
					}
				}
			free(str);
			}
		}
	}

	ifs.close();
	return true;
}

bool Config::Save() {
	std::string CfgFilename;
	std::ofstream ofs;

	CfgFilename=UserDataDir+std::string("/")+std::string(PACKAGE)+std::string(".conf");
	Util::ShellExpand(&CfgFilename);
	ofs.open(CfgFilename.c_str());

	if (ofs.fail()) {
		std::cout<<"Unable to save "<<CfgFilename<<"."<<std::endl;
		return false;
	}

	/* Actually save the config. values */
	ofs<<"ConfigVer="<<CONFIG_DEFAULT_ConfigVer<<std::endl;

	ofs<<"GPSSource="<<GPSSource_text[GPSSource]<<std::endl;
	ofs<<"AutoStartgpsd="<<AutoStartgpsd<<std::endl;
	ofs<<"AutoSelectChart="<<AutoSelectChart<<std::endl;
	ofs<<"gpsdHost="<<gpsdHost<<std::endl;
	ofs<<"gpsdPort="<<gpsdPort<<std::endl;
	ofs<<"gpsdCmd="<<gpsdCmd<<std::endl;

	ofs<<"SerialPortDevice="<<SerialPortDevice<<std::endl;
	ofs<<"SerialPortBaud="<<SerialPortBaud<<std::endl;
	ofs<<"AutoActivateGPS="<<AutoActivateGPS<<std::endl;
	ofs<<"CenterOnGPS="<<CenterOnGPS<<std::endl;
	ofs<<"gpsd="<<gpsd<<std::endl;
	ofs<<"GPSUpdateFrequency="<<GPSUpdateFrequency<<std::endl;

	ofs<<"PDFViewer="<<PDFViewer<<std::endl;
	ofs<<"ImageViewer="<<ImageViewer<<std::endl;

	ofs<<"SaveConfigOnExit="<<SaveConfigOnExit<<std::endl;
	ofs<<"PackageDataDir="<<PackageDataDir<<std::endl;
	ofs<<"UserDataDir="<<UserDataDir<<std::endl;
	ofs<<"DataStoreDir="<<DataStoreDir<<std::endl;
	ofs<<"dTPPImportDir="<<dTPPImportDir<<std::endl;
	ofs<<"SectionalsImportDir="<<SectionalsImportDir<<std::endl;
	ofs<<"DigitalAeroInfoImportDir="<<DigitalAeroInfoImportDir<<std::endl;
	ofs<<"NationalFlightDBImportDir="<<NationalFlightDBImportDir<<std::endl;
	ofs<<"XPlane900DataImportDir="<<XPlane900DataImportDir<<std::endl;
	ofs<<"METARSourceURL="<<METARSourceURL<<std::endl;

	ofs.setf(std::ios::fixed);
	ofs<<std::setprecision(5);
	ofs<<"GPSSimLon="<<GPSSimLon<<std::endl;
	ofs<<"GPSSimLat="<<GPSSimLat<<std::endl;
	ofs<<std::setprecision(1);
	ofs<<"GPSSimSpd="<<GPSSimSpd<<std::endl;
	ofs<<std::setprecision(0);
	ofs<<"GPSSimTrk="<<GPSSimTrk<<std::endl;
	ofs<<"GPSSimAlt="<<GPSSimAlt<<std::endl;
	ofs<<std::setprecision(6);
	ofs.unsetf(std::ios::fixed);

	ofs<<"DisableSS="<<DisableSS<<std::endl;
	ofs<<"DisablePM="<<DisablePM<<std::endl;

	ofs<<"ChartScale="<<ChartScale<<std::endl;
	ofs<<"ChartXOffset="<<ChartXOffset<<std::endl;
	ofs<<"ChartYOffset="<<ChartYOffset<<std::endl;
	ofs<<"OpenChartFile="<<OpenChartFile<<std::endl;
	ofs<<"OpenRouteFile="<<OpenRouteFile<<std::endl;
	ofs<<"OpenAircraftFile="<<OpenAircraftFile<<std::endl;

	ofs<<"ShowWelcomeOnStartup="<<ShowWelcomeOnStartup<<std::endl;

	ofs<<"METARHistory="<<METARHistory<<std::endl;
	ofs<<"ShapeFile="<<ShapeFile<<std::endl;

	ofs.close();
	return true;
}

void Config::SetDefaults() {
	ConfigVer=CONFIG_DEFAULT_ConfigVer;
	GPSSource=CONFIG_DEFAULT_GPSSource;
	SerialPortDevice=CONFIG_DEFAULT_SerialPortDevice;
	SerialPortBaud=CONFIG_DEFAULT_SerialPortBaud;
	AutoStartgpsd=CONFIG_DEFAULT_AutoStartgpsd;
	AutoActivateGPS=CONFIG_DEFAULT_AutoActivateGPS;
	PDFViewer=CONFIG_DEFAULT_PDFViewer;
	ImageViewer=CONFIG_DEFAULT_ImageViewer;
	SaveConfigOnExit=CONFIG_DEFAULT_SaveConfigOnExit;
	CenterOnGPS=CONFIG_DEFAULT_CenterOnGPS;
	gpsdHost=CONFIG_DEFAULT_gpsdHost;
	gpsdPort=CONFIG_DEFAULT_gpsdPort;
	gpsdCmd=CONFIG_DEFAULT_gpsdCmd;
	GPSUpdateFrequency=CONFIG_DEFAULT_GPSUpdateFrequency;
	AutoSelectChart=CONFIG_DEFAULT_AutoSelectChart;
	UserDataDir=CONFIG_DEFAULT_UserDataDir;
	PackageDataDir=CONFIG_DEFAULT_PackageDataDir;
	DataStoreDir=CONFIG_DEFAULT_DataStoreDir;
	dTPPImportDir=CONFIG_DEFAULT_dTPPImportDir;
	SectionalsImportDir=CONFIG_DEFAULT_SectionalsImportDir;
	DigitalAeroInfoImportDir=CONFIG_DEFAULT_DigitalAeroInfoImportDir;
	XPlane900DataImportDir=CONFIG_DEFAULT_XPlane900DataImportDir;
	METARSourceURL=CONFIG_DEFAULT_METARSourceURL;
	DisableSS=CONFIG_DEFAULT_Disable_SS;
	DisablePM=CONFIG_DEFAULT_Disable_PM;
	GPSSimLon=CONFIG_DEFAULT_GPS_SimLon;
	GPSSimLat=CONFIG_DEFAULT_GPS_SimLat;
	GPSSimTrk=CONFIG_DEFAULT_GPS_SimTrk;
	GPSSimSpd=CONFIG_DEFAULT_GPS_SimSpd;
	GPSSimAlt=CONFIG_DEFAULT_GPS_SimAlt;
	ChartScale=CONFIG_DEFAULT_ChartScale;
	ChartXOffset=CONFIG_DEFAULT_ChartXOffset;
	ChartYOffset=CONFIG_DEFAULT_ChartYOffset;
	OpenChartFile=CONFIG_DEFAULT_OpenChartFile;
	OpenAircraftFile=CONFIG_DEFAULT_OpenAircraftFile;
	ShowWelcomeOnStartup=CONFIG_DEFAULT_ShowWelcomeOnStartup;
	METARHistory=CONFIG_DEFAULT_METARHistory;
	ShapeFile=CONFIG_DEFAULT_ShapeFile;
}

int Config::GetConfigVer() {
	return ConfigVer;
}

void Config::SetConfigVer(int value) {
	ConfigVer=value;
}

enum Config::GPSSource_enum Config::GetGPSSource() {
	return GPSSource;
}

void Config::SetGPSSource(enum GPSSource_enum value) {
	GPSSource=value;
}

bool Config::SetGPSSource(std::string value) {
	int i;
	for (i=0;strlen(GPSSource_text[i]);i++)
	{
		if (strcasecmp(value.c_str(),GPSSource_text[i])==0)
		{
			GPSSource=(enum GPSSource_enum)i;
			break;
		}
	}

	return (strlen(GPSSource_text[i])!=0);
}

std::string Config::GetSerialPortDevice() {
	return SerialPortDevice;
}

void Config::SetSerialPortDevice(std::string value) {
	SerialPortDevice=value;
}

int Config::GetSerialPortBaud() {
	return SerialPortBaud;
}

void Config::SetSerialPortbaud(int value) {
	SerialPortBaud=value;
}

bool Config::GetAutoActivateGPS() {
	return AutoActivateGPS;
}

void Config::SetAutoActivateGPS(bool value) {
	AutoActivateGPS=value;
}

bool Config::GetAutoStartgpsd() {
	return AutoStartgpsd;
}

void Config::SetAutoStartgpsd(bool value) {
	AutoStartgpsd=value;
}

std::string Config::GetPDFViewer() {
	return PDFViewer;
}

void Config::SetPDFViewer(std::string value) {
	PDFViewer=value;
}

std::string Config::GetImageViewer() {
	return ImageViewer;
}

void Config::SetImageViewer(std::string value) {
	ImageViewer=value;
}

bool Config::GetDisableSS() {
	return DisableSS;
}

void Config::SetDisableSS(bool value) {
	DisableSS=value;
}

bool Config::GetDisablePM() {
	return DisablePM;
}

void Config::SetDisablePM(bool value) {
	DisablePM=value;
}

bool Config::GetSaveConfigOnExit() {
	return SaveConfigOnExit;
}

void Config::SetSaveConfigOnExit(bool value) {
	SaveConfigOnExit=value;
}

std::string Config::GetUserDataDir() {
	return UserDataDir;
}

void Config::SetUserDataDir(std::string value) {
	UserDataDir=value;
}

std::string Config::GetDataStoreDir() {
	return DataStoreDir;
}

void Config::SetDataStoreDir(std::string value) {
	DataStoreDir=value;
}

std::string Config::GetPackageDataDir() {
	return PackageDataDir;
}

std::string Config::GetMETARSourceURL() {
	return METARSourceURL;
}

void Config::SetMETARSourceURL(std::string value) {
	METARSourceURL=value;
}

bool Config::GetCenterOnGPS() {
	return CenterOnGPS;
}

void Config::SetCenterOnGPS(bool value) {
	CenterOnGPS=value;
}

std::string Config::GetgpsdHost() {
	return gpsdHost;
}

void Config::SetgpsdHost(std::string value) {
	gpsdHost=value;
}

std::string Config::GetgpsdPort() {
	return gpsdPort;
}

void Config::SetgpsdPort(std::string value) {
	gpsdPort=value;
}

std::string Config::GetgpsdCmd() {
	return gpsdCmd;
}

void Config::SetgpsdCmd(std::string value) {
	gpsdCmd=value;
}

time_t Config::GetGPSUpdateFrequency() {
	return GPSUpdateFrequency;
}

void Config::SetGPSUpdateFrequency(unsigned int value) {
	GPSUpdateFrequency=value;
}

std::string Config::GetdTPPImportDir() {
	return dTPPImportDir;
}

void Config::SetdTPPImportDir(std::string value) {
	dTPPImportDir=value;
}

std::string Config::GetSectionalsImportDir() {
	return SectionalsImportDir;
}

void Config::SetSectionalsImportDir(std::string value) {
	SectionalsImportDir=value;
}

std::string Config::GetDigitalAeroInfoImportDir() {
	return DigitalAeroInfoImportDir;
}

void Config::SetDigitalAeroInfoImportDir(std::string value) {
	DigitalAeroInfoImportDir=value;
}

std::string Config::GetNationalFlightDBImportDir() {
	return NationalFlightDBImportDir;
}

void Config::SetNationalFlightDBImportDir(std::string value) {
	NationalFlightDBImportDir=value;
}

std::string Config::GetXPlane900DataImportDir() {
	return XPlane900DataImportDir;
}

void Config::SetXPlane900DataImportDir(std::string value) {
	XPlane900DataImportDir=value;
}

bool Config::GetAutoSelectChart()
{
	return AutoSelectChart;
}

void Config::SetAutoSelectChart(bool value)
{
	AutoSelectChart=value;
}

void Config::SetGPSSimLon(double value)
{
	GPSSimLon=value;
}

double Config::GetGPSSimLon()
{
	return GPSSimLon;
}

void Config::SetGPSSimLat(double value)
{
	GPSSimLat=value;
}

double Config::GetGPSSimLat()
{
	return GPSSimLat;
}

void Config::SetGPSSimTrk(double value)
{
	GPSSimTrk=value;
}

double Config::GetGPSSimTrk()
{
	return GPSSimTrk;
}

void Config::SetGPSSimSpd(double value)
{
	GPSSimSpd=value;
}

double Config::GetGPSSimSpd()
{
	return GPSSimSpd;
}

void Config::SetGPSSimAlt(double value)
{
	GPSSimAlt=value;
}

double Config::GetGPSSimAlt()
{
	return GPSSimAlt;
}

void Config::SetChartScale(double value) {
	ChartScale=value;
}

double Config::GetChartScale() {
	return ChartScale;
}

void Config::SetChartXOffset(int value) {
	ChartXOffset=value;
}

int Config::GetChartXOffset() {
	return ChartXOffset;
}

void Config::SetChartYOffset(int value) {
	ChartYOffset=value;
}

int Config::GetChartYOffset() {
	return ChartYOffset;
}

void Config::SetOpenChartFile(std::string value) {
	OpenChartFile=value;
}

std::string Config::GetOpenChartFile() {
	return OpenChartFile;
}

void Config::SetOpenRouteFile(std::string value) {
	OpenRouteFile=value;
}

std::string Config::GetOpenRouteFile() {
	return OpenRouteFile;
}

void Config::SetOpenAircraftFile(std::string value) {
	OpenAircraftFile=value;
}

std::string Config::GetOpenAircraftFile() {
	return OpenAircraftFile;
}

bool Config::GetShowWelcomeOnStartup() {
	return ShowWelcomeOnStartup;
}

void Config::SetShowWelcomeOnStartup(bool value) {
	ShowWelcomeOnStartup=value;
}

int Config::GetMETARHistory() {
	return METARHistory;
}

void Config::SetMETARHistory(int value) {
	if (value<2)
		value=2;
	if (value>24)
		value=24;
	METARHistory=value;
}

std::string Config::GetShapeFile() {
	return ShapeFile;
}

void Config::SetShapeFile(std::string value) {
	ShapeFile=value;
}

bool Config::MkBool(std::string value) {
	if (value.length()==0)
		return false;
	if (value=="1")
		return true;
	if (value=="0")
		return false;
	if (toupper(value[0])=='T')
		return true;
	if (toupper(value[0])=='F')
		return false;
	if (toupper(value[0])=='Y')
		return true;
	if (toupper(value[0])=='N')
		return false;

	return false;
}
