/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <cstring>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef WIN32
	#include <termios.h>
	#include <sys/ioctl.h>
#endif

#include "Config.h"
#include "Exception.h"
#include "Util.h"
#include "SerialPort.h"

SerialPort::SerialPort(std::string Port, unsigned int Baud) {
	int BaudCode=0;
	Handle=0;

	this->Port=Port;
	this->Baud=Baud;

	switch (Baud) {
#ifndef WIN32
		case 230400	: BaudCode=B230400; break;
		case 115200	: BaudCode=B115200; break;
		case 57600	: BaudCode=B57600; break;
		case 38400	: BaudCode=B38400; break;
		case 19200	: BaudCode=B19200; break;
		case 9600	: BaudCode=B9600; break;
		case 4800	: BaudCode=B4800; break;
		case 2400	: BaudCode=B2400; break;
		case 1200	: BaudCode=B1200; break;
		case 300	: BaudCode=B300; break;
#else
		case 256000	: BaudCode=CBR_256000; break;
		case 128000	: BaudCode=CBR_128000; break;
		case 115200	: BaudCode=CBR_115200; break;
		case 57600	: BaudCode=CBR_57600; break;
		case 38400	: BaudCode=CBR_38400; break;
		case 19200	: BaudCode=CBR_19200; break;
		case 9600	: BaudCode=CBR_9600; break;
		case 4800	: BaudCode=CBR_4800; break;
		case 2400	: BaudCode=CBR_2400; break;
		case 1200	: BaudCode=CBR_1200; break;
		case 300	: BaudCode=CBR_300; break;
#endif
		default		:
			throw Exception("Invalid baud rate: "+Baud,
				Exception::LEVEL_ERROR);
	}

#ifndef WIN32
	Handle=open(Port.c_str(),O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (Handle==-1) {
		throw Exception(std::string("Unable to open serial port: ")+Util::StrError(),Exception::LEVEL_ERROR);
	}

	struct termios options;
	memset(&options,0,sizeof(options));
	cfmakeraw(&options);

	cfsetispeed(&options,BaudCode);
	cfsetospeed(&options,BaudCode);
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CRTSCTS;
	options.c_iflag = 0;
	options.c_lflag = 0;
	options.c_oflag = 0;
	options.c_cc[VMIN]=0;
	options.c_cc[VTIME]=0;
	options.c_cflag |= (CLOCAL|CREAD);
	if (tcsetattr(Handle,TCSAFLUSH,&options)!=0) {
		Cleanup();
		throw Exception(std::string("Unable to configure serial port. (")+Util::StrError()+")",Exception::LEVEL_ERROR);
	}

#else

	Handle=CreateFile(
		Port.c_str(),
		GENERIC_READ | GENERIC_WRITE,
		0,
		0,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		0
		);

	if(Handle==INVALID_HANDLE_VALUE){
		if(GetLastError()==ERROR_FILE_NOT_FOUND) {
			throw Exception("Unable to open serial port: device not found.",Exception::LEVEL_ERROR);
		}
		throw Exception(std::string("Unable to open serial port. (")+Util::StrError()+")",Exception::LEVEL_ERROR);
	}

	DCB dcbSerialParams = {0};
	dcbSerialParams.DCBlength=sizeof(dcbSerialParams);
	if (!GetCommState(Handle, &dcbSerialParams)) {
		Cleanup();
		throw Exception(std::string("Unable to configure serial port. (")+Util::StrError()+")",Exception::LEVEL_ERROR);
	}
	dcbSerialParams.BaudRate=BaudCode;
	dcbSerialParams.ByteSize=8;
	dcbSerialParams.StopBits=ONESTOPBIT;
	dcbSerialParams.Parity=NOPARITY;
	if(!SetCommState(Handle, &dcbSerialParams)){
		Cleanup();
		throw Exception(std::string("Unable to set up serial port. (")+Util::StrError()+")",Exception::LEVEL_ERROR);
	}

	COMMTIMEOUTS timeouts={0};
	timeouts.ReadIntervalTimeout=MAXDWORD;
	timeouts.ReadTotalTimeoutConstant=0;
	timeouts.ReadTotalTimeoutMultiplier=0;
	timeouts.WriteTotalTimeoutConstant=50;
	timeouts.WriteTotalTimeoutMultiplier=10;
	if(!SetCommTimeouts(Handle, &timeouts)){
		Cleanup();
		throw Exception(std::string("Unable to set serial timeouts. (")+Util::StrError()+")",Exception::LEVEL_ERROR);
	}

#endif
}

void SerialPort::Cleanup() {
	if (Handle) {
#ifndef WIN32
		close(Handle);
#else
		CloseHandle(Handle);
#endif
	}
}

SerialPort::~SerialPort() {
	Cleanup();
}
